@@ -32,7 +32,16 @@ from .utilities import check_extra, prettify_pydantic
 from .models import FOLDER_REGEX, NAME_REGEX
 from .config.user import SimvueConfiguration
 from .api.request import get_json_from_response
-from .api.objects import Run, Folder, Tag, Artifact, Alert
+from .api.objects import (
+    Run,
+    Folder,
+    Tag,
+    Artifact,
+    Alert,
+    FileArtifact,
+    ObjectArtifact,
+    get_folder_from_path,
+)
 
 
 CONCURRENT_DOWNLOADS = 10
@@ -42,8 +51,10 @@ logger = logging.getLogger(__file__)
 
 
 def _download_artifact_to_file(
-    artifact: Artifact, output_dir: pathlib.Path | None
+    artifact: FileArtifact | ObjectArtifact, output_dir: pathlib.Path | None
 ) -> None:
+    if not artifact.name:
+        raise RuntimeError(f"Expected artifact '{artifact.id}' to have a name")
     _output_file = (output_dir or pathlib.Path.cwd()).joinpath(artifact.name)
     # If this is a hierarchical structure being downloaded, need to create directories
     _output_file.parent.mkdir(parents=True, exist_ok=True)
@@ -174,14 +185,16 @@ class Client:
         self,
         filters: list[str] | None,
         *,
-        system: bool = False,
+        attributes: list[str] | None = None,
+        metadata: bool = False,
         metrics: bool = False,
         alerts: bool = False,
-        metadata: bool = False,
+        system_info: bool = False,
+        timing_info: bool = False,
         output_format: typing.Literal["dict", "objects", "dataframe"] = "objects",
         count_limit: pydantic.PositiveInt | None = 100,
         start_index: pydantic.NonNegativeInt = 0,
-        show_shared: bool = False,
+        show_shared: bool = True,
         sort_by_columns: list[tuple[str, bool]] | None = None,
     ) -> DataFrame | typing.Generator[tuple[str, Run], None, None] | None:
         """Retrieve all runs matching filters.
@@ -191,6 +204,9 @@ class Client:
         filters: list[str] | None
             set of filters to apply to query results. If None is specified
             return all results without filtering.
+        attributes : list[str] | None, optional
+            specify specific attributes to retrieve for runs.
+            Default of None includes all.
         metadata : bool, optional
             whether to include metadata information in the response.
             Default False.
@@ -200,6 +216,12 @@ class Client:
         alerts : bool, optional
             whether to include alert information in the response.
             Default False.
+        system_info : bool, optional
+            whether to include system information in the response.
+            Default False.
+        timing_info: bool, optional
+            whether to include timing information in the response.
+            Default False.
         output_format : Literal['dict', objects', 'dataframe'], optional
             the structure of the response
                 * dict - dictionary of values.
@@ -210,7 +232,7 @@ class Client:
         start_index : int, optional
             the index from which to count entries. Default is 0.
         show_shared : bool, optional
-            whether to include runs shared with the current user. Default is False.
+            whether to include runs shared with the current user. Default is True.
         sort_by_columns : list[tuple[str, bool]], optional
             sort by columns in the order given,
             list of tuples in the form (column_name: str, sort_descending: bool),
@@ -234,17 +256,20 @@ class Client:
         RuntimeError
             if there was a failure in data retrieval from the server
         """
+        filters = filters or []
         if not show_shared:
-            filters = (filters or []) + ["user == self"]
+            filters += ["user == self"]
 
         _runs = Run.get(
             count=count_limit,
             offset=start_index,
+            attributes=json.dumps(attributes),
             filters=json.dumps(filters),
             return_basic=True,
+            return_system=system_info,
+            return_timing=timing_info,
             return_metrics=metrics,
             return_alerts=alerts,
-            return_system=system,
             return_metadata=metadata,
             sorting=[dict(zip(("column", "descending"), a)) for a in sort_by_columns]
             if sort_by_columns
@@ -254,34 +279,8 @@ class Client:
         if output_format == "objects":
             return _runs
 
-        _params: dict[str, bool | str] = {
-            "filters": json.dumps(filters),
-            "return_basic": True,
-            "return_metrics": metrics,
-            "return_alerts": alerts,
-            "return_system": system,
-            "return_metadata": metadata,
-        }
-
-        response = requests.get(
-            f"{self._user_config.server.url}/runs",
-            headers=self._headers,
-            params=_params,
-        )
-
-        response.raise_for_status()
-
-        if output_format not in ("dict", "dataframe"):
-            raise ValueError("Invalid format specified")
-
-        json_response = get_json_from_response(
-            expected_status=[http.HTTPStatus.OK],
-            scenario="Run retrieval",
-            response=response,
-        )
-
-        if (response_data := json_response.get("data")) is None:
-            raise RuntimeError("Failed to retrieve runs data")
+        run_objs: list[Run] = [run for _, run in _runs]
+        response_data = [run.to_dict() for run in run_objs]
 
         if output_format == "dict":
             return response_data
@@ -347,10 +346,18 @@ class Client:
         _ids = Folder.ids(filters=json.dumps([f"path == {path}"]))
 
         try:
-            return next(_ids)
+            _id = next(_ids)
         except StopIteration:
             return None
 
+        with contextlib.suppress(StopIteration):
+            next(_ids)
+            raise RuntimeError(
+                f"Expected single folder match for '{path}', but found duplicate."
+            )
+
+        return _id
+
     @prettify_pydantic
     @pydantic.validate_call
     def delete_runs(
@@ -419,13 +426,17 @@ class Client:
             if allow_missing:
                 return None
             else:
-                raise RuntimeError(
-                    f"Deletion of folder '{folder_path}' failed, folder does not exist."
+                raise ObjectNotFoundError(
+                    name=folder_path,
+                    obj_type="folder",
                 )
         _response = Folder(identifier=folder_id).delete(
             delete_runs=remove_runs, recursive=recursive, runs_only=False
         )
 
+        if folder_id not in _response.get("folders", []):
+            raise RuntimeError("Deletion of folder failed, server returned mismatch.")
+
         return _response.get("runs", [])
 
     @prettify_pydantic
@@ -474,13 +485,14 @@ class Client:
         )  # type: ignore
 
     def _retrieve_artifacts_from_server(
-        self, run_id: str, name: str, count: int | None = None
-    ) -> typing.Generator[tuple[str, Artifact], None, None]:
-        return Artifact.get(
-            runs=json.dumps([run_id]),
-            filters=json.dumps([f"name == {name}"]),
-            count=count,
-        )  # type: ignore
+        self, run_id: str, name: str
+    ) -> FileArtifact | ObjectArtifact | None:
+        return Artifact.from_name(
+            run_id=run_id,
+            name=name,
+            server_url=self._user_config.server.url,
+            server_token=self._user_config.server.token,
+        )
 
     @prettify_pydantic
     @pydantic.validate_call
@@ -528,12 +540,14 @@ class Client:
         RuntimeError
             if retrieval of artifact from the server failed
         """
-        _artifact = Artifact.from_name(
-            run_id=run_id,
-            name=name,
-            server_url=self._user_config.server.url,
-            server_token=self._user_config.server.token,
-        )
+        _artifact = self._retrieve_artifacts_from_server(run_id, name)
+
+        if not _artifact:
+            raise ObjectNotFoundError(
+                obj_type="artifact",
+                name=name,
+                extra=f"for run '{run_id}'",
+            )
 
         _content = b"".join(_artifact.download_content())
 
@@ -573,12 +587,14 @@ class Client:
             if there was a failure during retrieval of information from the
             server
         """
-        _artifacts = self._retrieve_artifacts_from_server(run_id, name, count=1)
+        _artifact = self._retrieve_artifacts_from_server(run_id, name)
 
-        try:
-            _id, _artifact = next(_artifacts)
-        except StopIteration as e:
-            raise ValueError(f"No artifact '{name}' found for run '{run_id}'") from e
+        if not _artifact:
+            raise ObjectNotFoundError(
+                obj_type="artifact",
+                name=name,
+                extra=f"for run '{run_id}'",
+            )
 
         _download_artifact_to_file(_artifact, output_dir)
 
@@ -614,7 +630,9 @@ class Client:
             Artifact.from_run(run_id=run_id, category=category)
         )
 
-        with ThreadPoolExecutor(CONCURRENT_DOWNLOADS) as executor:
+        with ThreadPoolExecutor(
+            CONCURRENT_DOWNLOADS, thread_name_prefix=f"get_artifacts_run_{run_id}"
+        ) as executor:
             futures = [
                 executor.submit(_download_artifact_to_file, artifact, output_dir)
                 for _, artifact in _artifacts
@@ -657,17 +675,12 @@ class Client:
         RuntimeError
             if there was a failure when retrieving information from the server
         """
-        _folders: typing.Generator[tuple[str, Folder], None, None] = Folder.get(
-            filters=json.dumps([f"path == {folder_path}"])
-        )  # type: ignore
-
         try:
-            _, _folder = next(_folders)
-            if not read_only:
-                _folder.read_only(read_only)
-            return _folder
-        except StopIteration:
+            _folder = get_folder_from_path(path=folder_path)
+        except ObjectNotFoundError:
             return None
+        _folder.read_only(is_read_only=read_only)
+        return _folder
 
     @pydantic.validate_call
     def get_folders(
@@ -835,7 +848,8 @@ class Client:
 
         _args = {"filters": json.dumps(run_filters)} if run_filters else {}
 
-        _run_data = dict(Run.get(**_args))
+        if not run_ids:
+            _run_data = dict(Run.get(**_args))
 
         if not (
             _run_metrics := self._get_run_metrics_from_server(
@@ -853,7 +867,8 @@ class Client:
             )
         if use_run_names:
             _run_metrics = {
-                _run_data[key].name: _run_metrics[key] for key in _run_metrics.keys()
+                Run(identifier=key).name: _run_metrics[key]
+                for key in _run_metrics.keys()
             }
         return parse_run_set_metrics(
             _run_metrics,
@@ -1062,6 +1077,8 @@ class Client:
                     ]
                     if sort_by_columns
                     else None,
+                    count=count_limit,
+                    offset=start_index,
                 )
             ]  # type: ignore
 
@@ -1143,7 +1160,7 @@ class Client:
 
     @prettify_pydantic
     @pydantic.validate_call
-    def get_tag(self, tag_id: str) -> Tag | None:
+    def get_tag(self, tag_id: str) -> Tag:
         """Retrieve a single tag
 
         Parameters
@@ -1160,8 +1177,7 @@ class Client:
         ------
         RuntimeError
             if retrieval of information from the server on this tag failed
+        ObjectNotFoundError
+            if tag does not exist
         """
-        try:
-            return Tag(identifier=tag_id)
-        except ObjectNotFoundError:
-            return None
+        return Tag(identifier=tag_id)
