@@ -16,6 +16,7 @@ import sys
 import multiprocessing
 import threading
 import os
+import shutil
 import psutil
 import subprocess
 import contextlib
@@ -34,6 +35,15 @@ class CompletionCallback(typing.Protocol):
     def __call__(self, *, status_code: int, std_out: str, std_err: str) -> None: ...
 
 
+def get_current_shell() -> str | None:
+    """Return the users current shell executable."""
+    try:
+        _process = psutil.Process(os.getppid())
+        return os.environ.get("SHELL", _process.exe())
+    except psutil.Error:
+        return None
+
+
 def _execute_process(
     proc_id: str,
     command: list[str],
@@ -79,6 +89,8 @@ def _execute_process(
         thread_out = threading.Thread(
             target=trigger_check,
             args=(completion_callback, completion_trigger, _result),
+            daemon=True,
+            name=f"{proc_id}_Thread",
         )
         thread_out.start()
 
@@ -130,6 +142,33 @@ class Executor:
         with open(err_file) as err:
             return err.read() or None
 
+    @staticmethod
+    def _kwarg_assembly(kwargs, executable: str | None) -> list[str]:
+        _arguments: list[str] = []
+        _shell_is_pwsh: bool = any(
+            shell in get_current_shell() for shell in ("pwsh", "powershell")
+        )
+        _exec_is_pwsh: bool = executable in ("pwsh", "powershell", None)
+        _use_pwsh: bool = _shell_is_pwsh and _exec_is_pwsh
+
+        for arg, value in kwargs.items():
+            if arg.startswith("__"):
+                continue
+
+            arg = arg.replace("_", "-")
+
+            if len(arg) == 1 or _use_pwsh:
+                _arguments += (
+                    [f"-{arg}"]
+                    if isinstance(value, bool) and value
+                    else [f"-{arg}", f"{value}"]
+                )
+            elif isinstance(value, bool) and value:
+                _arguments += [f"--{arg}"]
+            else:
+                _arguments += [f"--{arg}", f"{value}"]
+        return _arguments
+
     def add_process(
         self,
         identifier: str,
@@ -205,6 +244,19 @@ class Executor:
                 "due to function pickling restrictions"
             )
 
+        # To check the executable provided by the user exists combine with environment
+        # PATH variable if exists, if not provided use the current environment
+        _session_path: str | None = (os.environ.copy() | (env or {})).get("PATH", None)
+
+        if (
+            executable
+            and not pathlib.Path(executable).exists()
+            and not shutil.which(executable, path=_session_path)
+        ):
+            raise FileNotFoundError(
+                f"Executable '{executable}' does not exist, please check the path/environment."
+            )
+
         if script:
             self._runner.save_file(file_path=script, category="code")
 
@@ -215,31 +267,19 @@ class Executor:
 
         if executable:
             command += [f"{executable}"]
-        else:
+        elif pos_args:
             command += [pos_args[0]]
             pos_args.pop(0)
+        elif executable := get_current_shell():
+            command += [f"{executable}"]
+
         if script:
             command += [f"{script}"]
 
         if input_file:
             command += [f"{input_file}"]
 
-        for arg, value in kwargs.items():
-            if arg.startswith("__"):
-                continue
-
-            arg = arg.replace("_", "-")
-
-            if len(arg) == 1:
-                command += (
-                    [f"-{arg}"]
-                    if isinstance(value, bool) and value
-                    else [f"-{arg}", f"{value}"]
-                )
-            elif isinstance(value, bool) and value:
-                command += [f"--{arg}"]
-            else:
-                command += [f"--{arg}", f"{value}"]
+        command += self._kwarg_assembly(kwargs, executable=executable)
 
         command += pos_args
 
@@ -377,7 +417,7 @@ class Executor:
             # This is so that if a process incorrectly reports its return code,
             # the user can manually set the correct status depending on logs etc.
             _alert = UserAlert(identifier=self._alert_ids[proc_id])
-            _is_set = _alert.get_status(run_id=self._runner._id)
+            _is_set = _alert.get_status(run_id=self._runner.id)
 
             if process.returncode != 0:
                 # If the process fails then purge the dispatcher event queue
@@ -404,6 +444,10 @@ class Executor:
 
     def _save_output(self) -> None:
         """Save the output to Simvue"""
+        if self._runner.status != "running":
+            logger.debug("Run is not active, skipping output save.")
+            return
+
         for proc_id in self._processes.keys():
             # Only save the file if the contents are not empty
             if self.std_err(proc_id):
