@@ -13,8 +13,8 @@ import threading
 import requests
 import psutil
 from simvue.config.user import SimvueConfiguration
-
 import simvue.api.objects
+from simvue.api.objects.artifact.base import ArtifactBase
 from simvue.eco.emissions_monitor import CO2Monitor
 from simvue.version import __version__
 
@@ -26,21 +26,40 @@ UPLOAD_ORDER: list[str] = [
     "tags",
     "alerts",
     "runs",
+    "grids",
     "artifacts",
     "metrics",
+    "grid_metrics",
     "events",
 ]
 
 _logger = logging.getLogger(__name__)
 
 
+def _log_upload_failed(file_path: pydantic.FilePath) -> None:
+    """Record that an object failed to upload in the object offline cache file.
+
+    Parameters
+    ----------
+    file_path : pydantic.FilePath
+        The path to the offline cache file for the object
+    """
+    with file_path.open("r") as file:
+        _data = json.load(file)
+    _data["upload_failed"] = True
+    with file_path.open("w") as file:
+        json.dump(_data, file)
+
+
 def upload_cached_file(
     cache_dir: pydantic.DirectoryPath,
     obj_type: str,
     file_path: pydantic.FilePath,
     id_mapping: dict[str, str],
+    throw_exceptions: bool,
+    retry_failed_uploads: bool,
     lock: threading.Lock,
-):
+) -> None:
     """Upload data stored in a cached file to the Simvue server.
 
     Parameters
@@ -53,46 +72,68 @@ def upload_cached_file(
         The path to the cached file to upload
     id_mapping : dict[str, str]
         A mapping of offline to online object IDs
+    throw_exceptions : bool
+        Whether to throw exceptions, or just log them
+    retry_failed_uploads : bool
+        Whether to retry failed uploads or ignore them
     lock : threading.Lock
         A lock to prevent multiple threads accessing the id mapping directory at once
     """
     _current_id = file_path.name.split(".")[0]
     _data = json.load(file_path.open())
     _exact_type: str = _data.pop("obj_type")
+
+    if _data.pop("upload_failed", False) and not retry_failed_uploads:
+        return
+
     try:
         _instance_class = getattr(simvue.api.objects, _exact_type)
-    except AttributeError as e:
-        raise RuntimeError(f"Attempt to initialise unknown type '{_exact_type}'") from e
+    except AttributeError as error:
+        if throw_exceptions:
+            raise error
+
+        _logger.error(f"Attempt to initialise unknown type '{_exact_type}'")
+        _log_upload_failed(file_path)
+        return
 
     # If it is an ObjectArtifact, need to load the object as bytes from a different file
     if issubclass(_instance_class, simvue.api.objects.ObjectArtifact):
         with open(file_path.parent.joinpath(f"{_current_id}.object"), "rb") as file:
             _data["serialized"] = file.read()
-
-    # We want to reconnect if there is an online ID stored for this file
-    if _online_id := id_mapping.get(_current_id):
-        obj_for_upload = _instance_class(
-            identifier=_online_id, _read_only=False, **_data
-        )
-    else:
-        obj_for_upload = _instance_class.new(**_data)
-
-    with lock:
-        obj_for_upload.on_reconnect(id_mapping)
-
     try:
-        obj_for_upload.commit()
+        # We want to reconnect if there is an online ID stored for this file
+        if _online_id := id_mapping.get(_current_id):
+            obj_for_upload = _instance_class(
+                identifier=_online_id, _read_only=False, **_data
+            )
+        else:
+            obj_for_upload = _instance_class.new(**_data)
+
+        with lock:
+            obj_for_upload.on_reconnect(id_mapping)
+
+        if not issubclass(_instance_class, ArtifactBase):
+            obj_for_upload.commit()
         _new_id = obj_for_upload.id
-    except RuntimeError as error:
-        if "status 409" in error.args[0]:
+
+    except Exception as error:
+        if "status 409" in str(error):
             return
-        raise error
-    if not _new_id:
-        raise RuntimeError(
-            f"Object of type '{obj_for_upload.__class__.__name__}' has no identifier"
+        if throw_exceptions:
+            raise error
+
+        _logger.error(
+            f"Error while committing '{_instance_class.__name__}': {str(error)}"
         )
+        _log_upload_failed(file_path)
+        return
+    if not _new_id:
+        _logger.error(f"Object of type '{_instance_class.__name__}' has no identifier")
+        _log_upload_failed(file_path)
+        return
+
     _logger.info(
-        f"{'Updated' if id_mapping.get(_current_id) else 'Created'} {obj_for_upload.__class__.__name__} '{_new_id}'"
+        f"{'Updated' if id_mapping.get(_current_id) else 'Created'} {_instance_class.__name__} '{_new_id}'"
     )
 
     file_path.unlink(missing_ok=True)
@@ -151,7 +192,8 @@ def sender(
     max_workers: int = 5,
     threading_threshold: int = 10,
     objects_to_upload: list[str] = UPLOAD_ORDER,
-    co2_intensity_refresh: int | None | str = None,
+    throw_exceptions: bool = False,
+    retry_failed_uploads: bool = False,
 ) -> dict[str, str]:
     """Send data from a local cache directory to the Simvue server.
 
@@ -165,9 +207,10 @@ def sender(
         The number of cached files above which threading will be used
     objects_to_upload : list[str]
         Types of objects to upload, by default uploads all types of objects present in cache
-    co2_intensity_refresh: int | None | str
-        the refresh interval for the CO2 intensity value, if None use config value if available,
-        else do not refresh.
+    throw_exceptions : bool, optional
+        Whether to throw exceptions as they are encountered in the sender, default is False (exceptions will be logged)
+    retry_failed_uploads : bool, optional
+        Whether to retry sending objects which previously failed, by default False
 
     Returns
     -------
@@ -203,19 +246,34 @@ def sender(
         _offline_files = _all_offline_files[_obj_type]
         if len(_offline_files) < threading_threshold:
             for file_path in _offline_files:
-                upload_cached_file(cache_dir, _obj_type, file_path, _id_mapping, _lock)
+                upload_cached_file(
+                    cache_dir=cache_dir,
+                    obj_type=_obj_type,
+                    file_path=file_path,
+                    id_mapping=_id_mapping,
+                    throw_exceptions=throw_exceptions,
+                    retry_failed_uploads=retry_failed_uploads,
+                    lock=_lock,
+                )
         else:
-            with ThreadPoolExecutor(max_workers=max_workers) as executor:
+            with ThreadPoolExecutor(
+                max_workers=max_workers, thread_name_prefix="sender_session_upload"
+            ) as executor:
                 _results = executor.map(
                     lambda file_path: upload_cached_file(
                         cache_dir=cache_dir,
                         obj_type=_obj_type,
                         file_path=file_path,
                         id_mapping=_id_mapping,
+                        throw_exceptions=throw_exceptions,
+                        retry_failed_uploads=retry_failed_uploads,
                         lock=_lock,
                     ),
                     _offline_files,
                 )
+                # This will raise any exceptions encountered during sending
+                for result in _results:
+                    pass
 
     # Send heartbeats
     _headers: dict[str, str] = {
@@ -234,7 +292,9 @@ def sender(
                 ),
             )
     else:
-        with ThreadPoolExecutor(max_workers=max_workers) as executor:
+        with ThreadPoolExecutor(
+            max_workers=max_workers, thread_name_prefix="sender_heartbeat"
+        ) as executor:
             _results = executor.map(
                 lambda _heartbeat_file: send_heartbeat(
                     file_path=_heartbeat_file,
@@ -249,17 +309,13 @@ def sender(
     # refreshes the CO2 intensity value if required. No emission metrics
     # will be taken by the sender itself, values are assumed to be recorded
     # by any offline runs being sent.
-
-    if (
-        _refresh_interval := co2_intensity_refresh
-        or _user_config.eco.intensity_refresh_interval
-    ):
+    if _user_config.metrics.enable_emission_metrics:
         CO2Monitor(
             thermal_design_power_per_gpu=None,
             thermal_design_power_per_cpu=None,
             local_data_directory=cache_dir,
-            intensity_refresh_interval=_refresh_interval,
-            co2_intensity=co2_intensity_refresh or _user_config.eco.co2_intensity,
+            intensity_refresh_interval=_user_config.eco.intensity_refresh_interval,
+            co2_intensity=_user_config.eco.co2_intensity,
             co2_signal_api_token=_user_config.eco.co2_signal_api_token,
         ).check_refresh()
 
